@@ -1,4 +1,3 @@
-
 #
 # NuGridpy - Tools for accessing and visualising NuGrid data.
 #
@@ -142,6 +141,24 @@ def set_nugrid_path(path):
     global nugrid_path
     nugrid_path=path
 
+def set_nice_params():
+    fsize=18
+        
+    params = {'axes.labelsize':  fsize,
+    #    'font.family':       'serif',
+    'font.family':        'Times New Roman',
+    'figure.facecolor':  'white',
+    'text.fontsize':     fsize,
+    'legend.fontsize':   fsize,
+    'xtick.labelsize':   fsize*0.8,
+    'ytick.labelsize':   fsize*0.8,
+    'ytick.minor.pad': 8,
+    'ytick.major.pad': 8,
+    'xtick.minor.pad': 8,
+    'xtick.major.pad': 8,
+    'text.usetex':       False}
+    pl.rcParams.update(params)
+
 class mesa_profile(DataPlot):
     ''' 
     read profiles.index and prepare reading MESA profile files
@@ -607,13 +624,106 @@ class history_data(DataPlot):
         column_array = self.data[:,self.cols[str_name]-1].astype('float')
         return column_array
 
-    def hrd(self):
-        ''' plot an HR diagram'''
+    def CO_ratio(self,ifig,ixaxis):
+        '''
+        plot surface C/O ratio in Figure ifig with x-axis quantity ixaxis
+            
+        Parameters
+        ----------
+        ifig : integer
+            Figure number in which to plot
+        ixaxis : string
+            what quantity is to be on the x-axis, either 'time' or 'model'
+            The default is 'model'
+        '''
+    
+        def C_O(model):
+            surface_c12=model.get('surface_c12')
+            surface_o16=model.get('surface_o16')
+            CORatio=(surface_c12*4.)/(surface_o16*3.)
+            return CORatio
 
-        pyl.plot(self.data[:,self.cols['log_Teff']-1],self.data[:,self.cols['log_L']-1],label = "M="+str(self.header_attr['initial_mass'])+", Z="+str(self.header_attr['initial_z']))
-        pyl.legend()
-        pyl.xlabel('log Teff')
-        pyl.ylabel('log L')
+        if ixaxis=='time':
+            xax=self.get('star_age')
+        elif ixaxis=='model':
+            xax=self.get('model_number')
+        else:
+            raise IOError("ixaxis not recognised")
+
+        pl.figure(ifig)
+        pl.plot(xax,CO(self))
+
+    def hrd(self,ifig=None,label=None,colour=None,s2ms=False):
+        '''
+        Plot an HR diagram
+        
+        Parameters
+        ----------
+        ifig : integer or string
+            Figure label, if None the current figure is used
+            The default value is None.
+        lims : list [x_lower, x_upper, y_lower, y_upper]
+        label : string
+            Label for the model
+            The default value is None
+        colour : string
+            The colour of the line
+            The default value is None
+        s2ms : boolean, optional
+            "Skip to Main Sequence"?
+            The default is False.
+        '''
+        
+        fsize=18
+        
+        params = {'axes.labelsize':  fsize,
+        #    'font.family':       'serif',
+        'font.family':        'Times New Roman',
+        'figure.facecolor':  'white',
+        'text.fontsize':     fsize,
+        'legend.fontsize':   fsize,
+        'xtick.labelsize':   fsize*0.8,
+        'ytick.labelsize':   fsize*0.8,
+        'text.usetex':       False}
+        pl.rcParams.update(params)
+        
+        if ifig is not None:
+            pl.figure(ifig)
+        
+        if s2ms:
+            h1=self.get('center_h1')
+            idx=np.where(h1[0]-h1>=1.e-3)[0][0]
+            skip=idx
+        else:
+            skip=0
+        
+        x = self.get('log_Teff')[skip:]
+        y = self.get('log_L')[skip:]
+        
+        if label is not None:
+            if colour is not None:
+                pl.plot(x,y,label=label,color=colour)
+            else:
+                pl.plot(x,y,label=label)
+            pl.legend(loc='best').draw_frame(False)
+        else:
+            if colour is not None:
+                pl.plot(x,y,color=colour)
+            else:
+                pl.plot(x,y)
+
+#        pyl.plot(self.data[:,self.cols['log_Teff']-1],\
+#                 self.data[:,self.cols['log_L']-1],\
+#                 label = "M="+str(self.header_attr['initial_mass'])+", Z="\
+#                 +str(self.header_attr['initial_z']))
+
+        pyl.xlabel('$\log T_{\\rm eff}$')
+        pyl.ylabel('$\log L$')
+        x1,x2=pl.xlim()
+        if x2 > x1:
+            ax=pl.gca()
+            ax.invert_xaxis()
+#            self._xlimrev()
 
     def hrd_key(self, key_str):
         ''' 
@@ -626,10 +736,14 @@ class history_data(DataPlot):
             
         '''
 
-        pyl.plot(self.data[:,self.cols['log_Teff']-1],self.data[:,self.cols['log_L']-1],label = key_str)
+        pyl.plot(self.data[:,self.cols['log_Teff']-1],\
+                 self.data[:,self.cols['log_L']-1],label = key_str)
         pyl.legend()
         pyl.xlabel('log Teff')
         pyl.ylabel('log L')
+        x1,x2=pl.xlim()
+        if x2 > x1:
+            self._xlimrev()
 
     def hrd_new(self, input_label="", skip=0):
         ''' 
@@ -662,13 +776,84 @@ class history_data(DataPlot):
         else:
             pyl.gca().set_xlim([max(xl_old+xl_new),min(xl_old+xl_new)])
 
-    def tcrhoc(self,lims=[3.,10.,8.,10.]):
+    def xche4_teff(self,ifig=None,lims=[1.,0.,3.4,4.7],label=None,colour=None,
+                   s2ms=True):
+        '''
+        Plot effective temperature against central helium abundance.
+        
+        Parameters
+        ----------
+        ifig : integer or string
+            Figure label, if None the current figure is used
+            The default value is None.
+        lims : list [x_lower, x_upper, y_lower, y_upper]
+        label : string
+        Label for the model
+            The default value is None
+        colour : string
+            The colour of the line
+            The default value is None
+        s2ms : boolean, optional
+            "Skip to Main Sequence"
+            The default is True
+        '''
+        fsize=18
+        params = {'axes.labelsize':  fsize,
+        #    'font.family':       'serif',
+        'font.family':        'Times New Roman',
+        'figure.facecolor':  'white',
+        'text.fontsize':     fsize,
+        'legend.fontsize':   fsize,
+        'xtick.labelsize':   fsize*0.8,
+        'ytick.labelsize':   fsize*0.8,
+        'text.usetex':       False}
+        pl.rcParams.update(params)
+        
+        
+        if s2ms:
+            h1=self.get('center_h1')
+            idx=np.where(h1[0]-h1>=1.e-3)[0][0]
+            skip=idx
+        else:
+            skip=0
+
+        x = self.get('center_he4')[skip:]
+        y = self.get('log_Teff')[skip:]    
+        if ifig is not None:
+            pl.figure(ifig)
+        if label is not None:
+            if colour is not None:
+                pl.plot(x,y,label=label,color=colour)
+            else:
+                pl.plot(x,y,label=label)
+            pl.legend(loc='best').draw_frame(False)
+        else:
+            if colour is not None:
+                pl.plot(x,y,color=colour)
+            else:
+                pl.plot(x,y)
+        pl.xlim(lims[:2])
+        pl.ylim(lims[2:])
+        pl.xlabel('$X_{\\rm c}(\,^4{\\rm He}\,)$')
+        pl.ylabel('$\log\,T_{\\rm eff}$')
+
+
+    def tcrhoc(self,ifig=None,lims=[3.,10.,8.,10.],label=None,colour=None):
         '''
         Central temperature again central density plot
             
         Parameters
         ----------
+        ifig : integer or string
+            Figure label, if None the current figure is used
+            The default value is None.
         lims : list [x_lower, x_upper, y_lower, y_upper]
+        label : string
+            Label for the model
+            The default value is None
+        colour : string
+            The colour of the line
+            The default value is None
         '''
 
         fsize=18
@@ -684,14 +869,168 @@ class history_data(DataPlot):
         'text.usetex':       False}
         pl.rcParams.update(params)
         
-        pl.figure()
-        pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),
-                 color='r')
+        if ifig is not None:
+            pl.figure(ifig)
+        
+        if label is not None:
+            if colour is not None:
+                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),label=label,
+                        color=colour)
+            else:
+                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),label=label)
+            pl.legend(loc='best').draw_frame(False)
+        else:
+            if colour is not None:
+                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),
+                        color=colour)
+            else:
+                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'))
+
         pl.xlim(lims[:2])
         pl.ylim(lims[2:])
         pl.xlabel('log $\\rho_{\\rm c}$')
         pl.ylabel('log $T_{\\rm c}$')
 
+    def mdot_t(self,ifig=None,lims=[7.4,2.6,-8.5,-4.5],label=None,colour=None):
+        '''
+        Plot mass loss history as a function of log-time-left
+        
+        Parameters
+        ----------
+        ifig : integer or string
+            Figure label, if None the current figure is used
+            The default value is None.
+        lims : list [x_lower, x_upper, y_lower, y_upper]
+        label : string
+            Label for the model
+            The default value is None
+        colour : string
+            The colour of the line
+            The default value is None
+        '''
+        
+        fsize=18
+        
+        params = {'axes.labelsize':  fsize,
+        #    'font.family':       'serif',
+        'font.family':        'Times New Roman',
+        'figure.facecolor':  'white',
+        'text.fontsize':     fsize,
+        'legend.fontsize':   fsize,
+        'xtick.labelsize':   fsize*0.8,
+        'ytick.labelsize':   fsize*0.8,
+        'text.usetex':       False}
+        pl.rcParams.update(params)
+        
+        if ifig is not None:
+            pl.figure(ifig)
+        
+        gage= self.get('star_age')
+        lage=np.zeros(len(gage))
+        agemin = max(abs(gage[-1]-gage[-2])/5.,1.e-10)
+        for i in np.arange(len(gage)):
+            if gage[-1]-gage[i]>agemin:
+                lage[i]=np.log10(gage[-1]-gage[i]+agemin)
+            else :
+                lage[i]=np.log10(agemin)
+        x = lage
+        y = self.get('log_abs_mdot')
+
+        if ifig is not None:
+            pl.figure(ifig)
+        if label is not None:
+            if colour is not None:
+                pl.plot(x,y,label=label,color=colour)
+            else:
+                pl.plot(x,y,label=label)
+            pl.legend(loc='best').draw_frame(False)
+        else:
+            if colour is not None:
+                pl.plot(x,y,color=colour)
+            else:
+                pl.plot(x,y)
+        pl.xlim(lims[:2])
+        pl.ylim(lims[2:])
+        pl.ylabel('$\mathrm{log}_{10}(\|\dot{M}\|/M_\odot\,\mathrm{yr}^{-1})$')
+        pl.xlabel('$\mathrm{log}_{10}(t^*/\mathrm{yr})$')
+
+    def mcc_t(self,ifig=None,lims=[0,15,0,25],label=None,colour=None,
+              mask=False):
+        '''
+        Plot mass of convective core as a function of time.
+        
+        Parameters
+        ----------
+        ifig : integer or string
+            Figure label, if None the current figure is used
+            The default value is None.
+        lims : list [x_lower, x_upper, y_lower, y_upper]
+        label : string
+            Label for the model
+            The default value is None
+        colour : string
+            The colour of the line
+            The default value is None
+        mask : boolean, optional
+            Do you want to try to hide numerical spikes in the
+            plot?
+            The default is False
+        '''
+        
+        fsize=18
+        
+        params = {'axes.labelsize':  fsize,
+        #    'font.family':       'serif',
+        'font.family':        'Times New Roman',
+        'figure.facecolor':  'white',
+        'text.fontsize':     fsize,
+        'legend.fontsize':   fsize,
+        'xtick.labelsize':   fsize*0.8,
+        'ytick.labelsize':   fsize*0.8,
+        'text.usetex':       False}
+        pl.rcParams.update(params)
+        
+        if ifig is not None:
+            pl.figure(ifig)
+        
+        age= self.get('star_age')
+        x1 = age / 1.e6
+        x2 = age / 1.e6
+        y1 = self.get('mix_qtop_1')*self.get('star_mass')
+        y2 = self.get('mix_qtop_2')*self.get('star_mass')
+        mt1 = self.get('mix_type_1')
+        mt2 = self.get('mix_type_2')
+        
+        # Mask spikes...
+        if mask:
+            x1 = np.ma.masked_where(mt1 != 1, x1)
+            x2 = np.ma.masked_where(mt2 != 1, x2)
+            y1 = np.ma.masked_where(mt1 != 1, y1)
+            y2 = np.ma.masked_where(mt2 != 1, y2)
+        
+        if ifig is not None:
+            pl.figure(ifig)
+        if label is not None:
+            if colour is not None:
+                pl.plot(x1,y1,label=label,color=colour)
+                pl.plot(x2,y2,color=colour)
+            else:
+                pl.plot(x1,y1,label=label)
+                pl.plot(x2,y2)
+            pl.legend(loc='best').draw_frame(False)
+        else:
+            if colour is not None:
+                pl.plot(x1,y1,color=colour)
+                pl.plot(x2,y2,color=colour)
+            else:
+                pl.plot(x1,y1)
+                pl.plot(x2,y2)
+        pl.xlim(lims[:2])
+        pl.ylim(lims[2:])
+        pl.ylabel('$M/M_\odot}$')
+        pl.xlabel('$t/{\\rm Myr}$')
+
+
     def kippenhahn_CO(self, num_frame, xax, t0_model=0,
                       title='Kippenhahn diagram', tp_agb=0.,
                       ylim_CO=[0,0]):
@@ -734,8 +1073,17 @@ class history_data(DataPlot):
 
         t0_mod=xaxisarray[t0_model]
 
-        h1_boundary_mass  = self.get('h1_boundary_mass')
-        he4_boundary_mass = self.get('he4_boundary_mass')
+        plot_bounds=True
+        try:
+            h1_boundary_mass  = self.get('h1_boundary_mass')
+            he4_boundary_mass = self.get('he4_boundary_mass')
+        except:
+            try:
+                h1_boundary_mass  = self.get('he_core_mass')
+                he4_boundary_mass = self.get('c_core_mass')
+            except:
+                plot_bounds=False
+
         star_mass         = self.get('star_mass')
         mx1_bot           = self.get('mx1_bot')*star_mass
         mx1_top           = self.get('mx1_top')*star_mass
@@ -757,8 +1105,9 @@ class history_data(DataPlot):
             pyl.xlabel('model number')
 
         pyl.twinx()
-        pyl.plot(xaxisarray[t0_model:]-t0_mod,h1_boundary_mass[t0_model:],label='h1_boundary_mass')
-        pyl.plot(xaxisarray[t0_model:]-t0_mod,he4_boundary_mass[t0_model:],label='he4_boundary_mass')
+        if plot_bounds:
+            pyl.plot(xaxisarray[t0_model:]-t0_mod,h1_boundary_mass[t0_model:],label='h1_boundary_mass')
+            pyl.plot(xaxisarray[t0_model:]-t0_mod,he4_boundary_mass[t0_model:],label='he4_boundary_mass')
         pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_bot[t0_model:],',r',label='conv bound')
         pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_top[t0_model:],',r')
         pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_bot[t0_model:],',r')
@@ -778,8 +1127,7 @@ class history_data(DataPlot):
                    title='Kippenhahn diagram', tp_agb=0., t_eps=5.e2,
                    plot_star_mass=True, symbol_size=8, c12_bm=False,
                    print_legend=True):
-        ''' 
-        Kippenhahn plot as a function of time or model.
+        '''Kippenhahn plot as a function of time or model.
 
         Parameters
         ----------
@@ -790,13 +1138,14 @@ class history_data(DataPlot):
             Either 'model', 'time' or 'logtimerev' to indicate what is
             to be used on the x-axis.
         t0_model : integer, optional
-            Model for the zero point in time, for AGB plots this would
-            be usually the model of the 1st TP, which can be found with
-            the Kippenhahn plot.  The default is 0.
+            If xax = 'time' then model for the zero point in time, for
+            AGB plots this would be usually the model of the 1st TP,
+            which can be found with the Kippenhahn plot.  The default
+            is 0.
         title : string, optional
             The figure title.  The default is "Kippenhahn diagram".
         tp_agb : float, optional
-            If >= 0 then,
+            If > 0. then,
             ylim=[h1_min*1.-tp_agb/100 : h1_max*1.+tp_agb/100] with
             h1_min, h1_max the min and max H-free core mass coordinate.
             The default is 0. .
@@ -811,7 +1160,7 @@ class history_data(DataPlot):
             If we plot c12_boundary_mass or not.  The default is False.
         print_legend : boolean, optionla
             Show or do not show legend.  The defalut is True.
-            
+
         '''
 
         if num_frame >= 0:
@@ -821,12 +1170,17 @@ class history_data(DataPlot):
 
         if xax == 'time':
             xaxisarray = self.get('star_age')
-            t0_mod=xaxisarray[t0_model]
+            if t0_model > 0:
+                ind=self.get('model_number')
+                t0_model=where(ind>t0_model)[0][0]
+                t0_mod=xaxisarray[t0_model]
+            else:
+                t0_mod = 0.
             print 'zero time is '+str(t0_mod)
         elif xax == 'model':
             xaxisarray = self.get('model_number')
             #t0_mod=xaxisarray[t0_model]
-            t0_mod = 0
+            t0_mod = 0.
         elif xax == 'logtimerev':
             xaxi    = self.get('star_age')
             xaxisarray = np.log10(np.max(xaxi)+t_eps-xaxi)
@@ -836,10 +1190,21 @@ class history_data(DataPlot):
                   ' needs to be "time" or "model"'
 
 
-        h1_boundary_mass  = self.get('h1_boundary_mass')
-        he4_boundary_mass = self.get('he4_boundary_mass')
-        if c12_bm:
-            c12_boundary_mass = self.get('c12_boundary_mass')
+        plot_bounds=True
+        try:
+            h1_boundary_mass  = self.get('h1_boundary_mass')
+            he4_boundary_mass = self.get('he4_boundary_mass')
+            if c12_bm:
+                c12_boundary_mass = self.get('c12_boundary_mass')
+        except:
+            try:
+                h1_boundary_mass  = self.get('he_core_mass')
+                he4_boundary_mass = self.get('c_core_mass')
+                if c12_bm:
+                    c12_boundary_mass = self.get('o_core_mass')
+            except:
+                plot_bounds=False
+
         star_mass         = self.get('star_mass')
         mx1_bot           = self.get('mx1_bot')*star_mass
         mx1_top           = self.get('mx1_top')*star_mass
@@ -861,10 +1226,12 @@ class history_data(DataPlot):
         pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_top[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size)
         pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_bot[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size)
         pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_top[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size)
-        pyl.plot(xaxisarray[t0_model:]-t0_mod,h1_boundary_mass[t0_model:],color='red',linewidth=2,label='H-free core')
-        pyl.plot(xaxisarray[t0_model:]-t0_mod,he4_boundary_mass[t0_model:],color='green',linewidth=2,linestyle='dashed',label='He-free core')
-        if c12_bm:
-            pyl.plot(xaxisarray[t0_model:]-t0_mod,c12_boundary_mass[t0_model:],color='purple',linewidth=2,linestyle='dotted',label='C-free core')
+
+        if plot_bounds:
+            pyl.plot(xaxisarray[t0_model:]-t0_mod,h1_boundary_mass[t0_model:],color='red',linewidth=2,label='H-free core')
+            pyl.plot(xaxisarray[t0_model:]-t0_mod,he4_boundary_mass[t0_model:],color='green',linewidth=2,linestyle='dashed',label='He-free core')
+            if c12_bm:
+                pyl.plot(xaxisarray[t0_model:]-t0_mod,c12_boundary_mass[t0_model:],color='purple',linewidth=2,linestyle='dotted',label='C-free core')
         if plot_star_mass is True:
             pyl.plot(xaxisarray[t0_model:]-t0_mod,star_mass[t0_model:],label='$M_\star$')
         pyl.ylabel('$m_\mathrm{r}/\mathrm{M}_\odot$')
@@ -1259,14 +1626,15 @@ class history_data(DataPlot):
         #fig.savefig(outfile)
         pl.show()
 
-    def kip_cont(self, ifig=110, modstart=0, modstop=-1,
+    def kip_cont(self, ifig=110, modstart=0, modstop=-1,t0_model=0,
                  outfile='out.png', xlims=[0.,0.], ylims=[0.,0.],
                  xres=1000, yres=1000, ixaxis='model_number',
                  mix_zones=20, burn_zones=20, plot_radius=False,
                  engenPlus=True, engenMinus=False,
                  landscape_plot=False, rad_lines=False, profiles=[],
                  showfig=True, outlines=True, boundaries=True,
-                 c12_boundary=False, rasterise=False, yscale='1.'):
+                 c12_boundary=False, rasterise=False, yscale='1.',
+                 engenlevels=None):
         ''' 
         This function creates a Kippenhahn plot with energy flux using
         contours.
@@ -1296,11 +1664,19 @@ class history_data(DataPlot):
             Model from which you want to plot (be careful if your
             history.data or star.log output is sparse...).  If it is
             0 then it starts from the beginning, works even if
-            log_cnt > 1.  The default is 0.
+            log_cnt > 1.
+            The default is 0.
         modstop : integer, optional
             Model to which you wish to plot, -1 corresponds to end
             [if log_cnt>1, devide modstart and modstop by log_cnt,
-            this needs to be improved!].  The defalut is -1.
+            this needs to be improved! SJ: this should be ficed now].
+            The defalut is -1.
+        t0_model : integer, optional
+            Model number from which to reset the time to 0. Typically,
+            if modstart!=0, t0_model=modstart is a good choice, but we
+            leave the choice to the user in case the time is wished to
+            start from 0 at a different key point of the evolution.
+            The default value is 0.
         outfile : sting, optional
             'filename + extension' where you want to save the figure.
             The defalut is "out.png".
@@ -1364,13 +1740,24 @@ class history_data(DataPlot):
             smaller vector graphics figures.  The default is False.
         yscale : string, optional
             Re-scale the y-axis by this amount
+        engenlevels : list
+            Give cusstom levels to the engenPlus contour. If None,
+            the levels are chosen automatically.
+            The default is None.
             
         Notes
         -----
         The parameter xlims is depricated.
         
         '''
-
+        
+        # Find correct modstart and modstop:
+        mod=np.array([int(i) for i in self.get('model_number')])
+        mod1=np.abs(mod-modstart).argmin()
+        mod2=np.abs(mod-modstop).argmin()
+        if modstart != 0 : modstart=mod1
+        if modstop != -1 : modstop=mod2
+        
         xxyy=[self.get('star_age')[modstart:modstop],self.get('star_age')[modstart:modstop]]
         mup = max(float(self.get('star_mass')[0])*1.02,1.0)
         nmodels=len(self.get('model_number')[modstart:modstop])
@@ -1551,8 +1938,9 @@ class history_data(DataPlot):
             if xlims[1] == 0.:
                 xlims = [self.get('model_number')[modstart],self.get('model_number')[modstop]]
         elif ixaxis =='age':
-            if modstart != 0:
-                xxx= self.get('star_age')[modstart:modstop] - self.get('star_age')[modstart]
+            if t0_model != 0:
+                t0_mod=np.abs(mod-t0_model).argmin()
+                xxx= self.get('star_age')[modstart:modstop] - self.get('star_age')[t0_mod]
                 print 'plot versus age'
                 ax.set_xlabel('Age [yr] - '+str(self.get('star_age')[modstart]),fontsize=fsize)
             else:
@@ -1586,13 +1974,13 @@ class history_data(DataPlot):
             ax.add_artist(cc)
             return cc
 
-#       cmapMIX = matplotlib.colors.ListedColormap(['w','k'])
         cmapMIX = matplotlib.colors.ListedColormap(['w','#8B8386']) # rose grey
-#        cmapMIX = matplotlib.colors.ListedColormap(['w','#23238E']) # navy blue
         cmapB1  = pyl.cm.get_cmap('Blues')
         cmapB2  = pl.cm.get_cmap('Reds')
-        #cmapB1  = matplotlib.colors.ListedColormap(['w','b'])
-        #cmapB2  = matplotlib.colors.ListedColormap(['r','w'])
+        
+        ylims1=[0.,0.]
+        ylims1[0]=ylims[0]
+        ylims1[1]=ylims[1]
         if ylims == [0.,0.]:
             ylims[0] = 0.
             ylims[1] = mup
@@ -1608,60 +1996,88 @@ class history_data(DataPlot):
             ylims[0] = y[0]
             ylims[1] = y[-1]
 
-#        print xxx[::dx]
-#        print y
-#        print Z
-
         print 'plotting contours'
         CMIX    = ax.contourf(xxx[::dx],y,Z, cmap=cmapMIX,alpha=0.6,levels=[0.5,1.5])
         if rasterise==True:
             insert_rasterized_contour_plot(CMIX)
-        #CMIX_outlines    = ax.contour(xxx[::dx],y,Z, cmap=cmapMIX, alpha=1.0,levels=[0.5,1.5])
-        #CMIX    = ax.contourf(xxx[::dx],y,Z, cmap=cmapMIX, alpha=0.5)
         if outlines == True:
             CMIX_outlines    = ax.contour(xxx[::dx],y,Z, cmap=cmapMIX)
             if rasterise==True:
                 insert_rasterized_contour_plot(CMIX_outlines)
 
         if engenstyle == 'full' and engenPlus == True:
-#            print B1
-#            print B2
-            CBURN1  = ax.contourf(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.5, locator=matplotlib.ticker.LogLocator())
-            CB1_outlines  = ax.contour(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.7, locator=matplotlib.ticker.LogLocator())
-            #CBURN1  = ax.contourf(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.5)
+            if engenlevels!= None:
+                CBURN1  = ax.contourf(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.5, locator=matplotlib.ticker.LogLocator(),levels=engenlevels)
+                if outlines:
+                    CB1_outlines  = ax.contour(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.7, locator=matplotlib.ticker.LogLocator(),levels=engenlevels)
+            else:
+                CBURN1  = ax.contourf(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.5, locator=matplotlib.ticker.LogLocator())
+                if outlines:
+                    CB1_outlines  = ax.contour(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.7, locator=matplotlib.ticker.LogLocator())
             CBARBURN1 = pyl.colorbar(CBURN1)
             CBARBURN1.set_label('$|\epsilon_\mathrm{nuc}-\epsilon_{\\nu}| \; (\mathrm{erg\,g}^{-1}\mathrm{\,s}^{-1})$',fontsize=fsize)
             if rasterise==True:
                 insert_rasterized_contour_plot(CBURN1)
+                if outlines:
+                    insert_rasterized_contour_plot(CB1_outlines)
+
         if engenstyle == 'full' and engenMinus == True:
             CBURN2  = ax.contourf(xxx[::dx],y,B2, cmap=cmapB2, alpha=0.5, locator=matplotlib.ticker.LogLocator())
-            CBURN2_outlines  = ax.contour(xxx[::dx],y,B2, cmap=cmapB2, alpha=0.7, locator=matplotlib.ticker.LogLocator())
-#               CBURN2  = ax.contourf(xxx[::dx],y,B2, cmap=cmapB2, alpha=0.5)
+            if outlines:
+                CBURN2_outlines  = ax.contour(xxx[::dx],y,B2, cmap=cmapB2, alpha=0.7, locator=matplotlib.ticker.LogLocator())
             CBARBURN2 = pl.colorbar(CBURN2)
             if engenPlus == False:
                 CBARBURN2.set_label('$|\epsilon_\mathrm{nuc}-\epsilon_{\\nu}| \; (\mathrm{erg\,g}^{-1}\mathrm{\,s}^{-1})$',fontsize=fsize)
             if rasterise==True:
                 insert_rasterized_contour_plot(CBURN2)
+                if outlines:
+                    insert_rasterized_contour_plot(CB2_outlines)
+
         if engenstyle == 'twozone' and (engenPlus == True or engenMinus == True):
             ax.contourf(xxx[::dx],y,V, cmap=cmapB1, alpha=0.5)
 
         print 'plotting patches'
-        ax.plot(xxx[::dx],self.get('star_mass')[modstart:modstop][::dx],'k-')
+        mtot=self.get('star_mass')[modstart:modstop][::dx]
+        mtot1=(mtot-ylims1[0])*float(yscale)
+        ax.plot(xxx[::dx],mtot,'k-')
 
         if boundaries == True:
             print 'plotting abund boundaries'
             try:
-                ax.plot(xxx,self.get('h1_boundary_mass')[modstart:modstop]-ylims[0],label='H boundary',linestyle='-')
-                ax.plot(xxx,self.get('he4_boundary_mass')[modstart:modstop]-ylims[0],label='He boundary',linestyle='--')
-                ax.plot(xxx,self.get('c12_boundary_mass')[modstart:modstop]-ylims[0],label='C boundary',linestyle='-.')
+                bound=self.get('h1_boundary_mass')[modstart:modstop]
+                bound1=(bound-ylims1[0])*float(yscale)
+                ax.plot(xxx,bound1,label='H boundary',linestyle='-')
+                
+                bound=self.get('he4_boundary_mass')[modstart:modstop]
+                bound1=(bound-ylims1[0])*float(yscale)
+                ax.plot(xxx,bound1,label='He boundary',linestyle='--')
+                
+                bound=self.get('c12_boundary_mass')[modstart:modstop]
+                bound1=(bound-ylims1[0])*float(yscale)
+                ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
 
             except:
                 try:
-                    ax.plot(xxx,self.get('he_core_mass')[modstart:modstop]-ylims[0],label='H boundary',linestyle='-')
-                    ax.plot(xxx,self.get('c_core_mass')[modstart:modstop]-ylims[0],label='He boundary',linestyle='--')
-                    ax.plot(xxx,self.get('o_core_mass')[modstart:modstop]-ylims[0],label='C boundary',linestyle='-.')
-                    ax.plot(xxx,self.get('si_core_mass')[modstart:modstop]-ylims[0],label='C boundary',linestyle='-.')
-                    ax.plot(xxx,self.get('fe_core_mass')[modstart:modstop]-ylims[0],label='C boundary',linestyle='-.')
+                    bound=self.get('he_core_mass')[modstart:modstop]
+                    bound1=(bound-ylims1[0])*float(yscale)
+                    ax.plot(xxx,bound1,label='H boundary',linestyle='-')
+                    
+                    bound=self.get('c_core_mass')[modstart:modstop]-ylims[0]
+                    bound1=(bound-ylims1[0])*float(yscale)
+                    ax.plot(xxx,bound1,label='He boundary',linestyle='--')
+                    
+                    bound=self.get('o_core_mass')[modstart:modstop]-ylims[0]
+                    bound1=(bound-ylims1[0])*float(yscale)
+                    ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
+                    
+                    bound=self.get('si_core_mass')[modstart:modstop]-ylims[0]
+                    bound1=(bound-ylims1[0])*float(yscale)
+                    ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
+                    
+                    bound=self.get('fe_core_mass')[modstart:modstop]-ylims[0]
+                    bound1=(bound-ylims1[0])*float(yscale)
+                    ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
+                
                 except:
                     pass
 
@@ -2025,6 +2441,10 @@ def _read_mesafile(filename,data_rows=0,only='all'):
 
     data = []
     for i in range(data_rows):
+        # writing reading status
+        percent = int(i*100/data_rows)
+        sys.stdout.flush()
+        sys.stdout.write("\r reading " + "...%d%%" % percent)
         line = f.readline()
         v=line.split()
         try:
@@ -2034,6 +2454,7 @@ def _read_mesafile(filename,data_rows=0,only='all'):
                 if item.__contains__('.') and not item.__contains__('E'):
                     v[v.index(item)]='0'
         data.append(vv)
+    print '\n'
     f.close()
     a=np.array(data)
     data = []
